/**
 * Stopwatch - Stops the time
 * <br> Use <code>start()</code> to start it and <code>stop()</code> to stop it. You can also reset it via
 * <code>reset()</code>.
 * <br> Check the Stopwatch status with: <code>hasStarted()</code>, <code>isRunning()</code> and <code>isStopped()</code>.
 * <br> Laps can be created with <code>lap(string)</code> and read with <code>getLaps()</code>, you can also
 * subscribe to <code>lapsChange</code>.
 * <br> The time can be read via <code>getTime()</code>, <code>getTimeInSeconds()</code>, <code>getTimeInMinutes()</code>
 * and <code>getTimeInHours()</code>.
 * @since 3.2.5
 */
export class Stopwatch {
    constructor(start = false) {
        this.laps = new Map();
        if (start) {
            this.start();
        }
    }
    hasStarted() {
        return !!this.startedAt;
    }
    isRunning() {
        return !!(this.startedAt && !this.endedAt);
    }
    isStopped() {
        return !!(this.startedAt && this.endedAt);
    }
    /**
     * Starts stopwatch (only once)
     * @return {Date} Date at which the stopwatch was started
     */
    start() {
        if (!this.startedAt) {
            this.startedAt = new Date();
        }
        return this.startedAt;
    }
    /**
     * Stops stopwatch (only once)
     * @return {Date} Date at which the stopwatch was stopped
     */
    stop() {
        if (!this.startedAt) {
            throw 'Stopwatch not started';
        }
        if (!this.endedAt) {
            this.endedAt = new Date();
        }
        return this.endedAt;
    }
    /**
     * Stops stopwatch and returns elapsed time in milliseconds
     * @return {number} Elapsed time in milliseconds
     */
    stopAndGetTime() {
        this.stop();
        return this.getTimeInSeconds();
    }
    /**
     * Stops stopwatch and returns elapsed time in seconds
     * @return {number} Elapsed time in seconds
     */
    stopAndGetTimeInSeconds() {
        this.stop();
        return this.getTimeInSeconds();
    }
    /**
     * Stops stopwatch and returns elapsed time in minutes
     * @return {number} Elapsed time in minutes
     */
    stopAndGetTimeInMinutes() {
        this.stop();
        return this.getTimeInMinutes();
    }
    /**
     * Stops stopwatch and returns elapsed time in hours
     * @return {number} Elapsed time in hours
     */
    stopAndGetTimeInHours() {
        this.stop();
        return this.getTimeInMinutes();
    }
    /**
     * Resets complete stopwatch
     */
    reset() {
        this.startedAt = undefined;
        this.endedAt = undefined;
        this.laps = new Map();
    }
    /**
     * @return {number} Elapsed time in milliseconds
     */
    getTime() {
        let endedAt = this.endedAt;
        if (!this.startedAt) {
            throw 'Stopwatch not started';
        }
        if (!endedAt) {
            endedAt = new Date();
        }
        return endedAt.getTime() - this.startedAt.getTime();
    }
    /**
     * @return {number} Elapsed time in seconds
     */
    getTimeInSeconds() {
        return this.getTime() / 1000;
    }
    /**
     * @return {number} Elapsed time in minutes
     */
    getTimeInMinutes() {
        return this.getTime() / 1000 / 60;
    }
    /**
     * @return {number} Elapsed time in hours
     */
    getTimeInHours() {
        return this.getTime() / 1000 / 60 / 60;
    }
    /**
     * Creates a lap with optional string, notifies all lap subscribers
     * @param {string} text
     * @return {Date} Date when the lap was set
     */
    lap(text) {
        if (!this.startedAt) {
            throw 'Stopwatch not started';
        }
        const date = new Date();
        this.laps.set(date.getTime(), text);
        return date;
    }
    /**
     * Returns all laps
     * @return {Map<number, UndefinedOr<string>>} All laps
     */
    getLaps() {
        return this.laps;
    }
    static createStarted() {
        return new Stopwatch(true);
    }
    static createStopped() {
        return new Stopwatch(false);
    }
}
//# sourceMappingURL=stopwatch.js.map